package aQute.maven;

import java.util.regex.*;

public class MavenVersion {
	public final static Pattern	MAVEN_VERSION	= Pattern
														.compile("(\\d+)(?:\\.(\\d+)(?:\\.(\\d+))?)?(?:-([-\\d\\w_.]+))?");

	final int					major;
	final int					minor;
	final int					increment;
	final String				qualifier;
	final String				all;
	final boolean				ok;

	public MavenVersion(String version) throws Exception {
		this.all = version;
		Matcher m = MAVEN_VERSION.matcher(version);
		if (m.matches()) {

			major = Integer.parseInt(m.group(1));
			if (m.group(2) != null)
				minor = Integer.parseInt(m.group(2));
			else
				minor = 0;

			if (m.group(3) != null)
				increment = Integer.parseInt(m.group(3));
			else
				increment = 0;

			if (m.group(4) != null)
				qualifier = m.group(4);
			else
				qualifier = null;

			ok = true;
		} else {
			major = 0;
			minor = 0;
			increment = 0;
			qualifier = null;
			ok = false;
		}
	}

	public MavenVersion(int major, int minor, int increment, String qualifier) {
		this.major = major;
		this.minor = minor;
		this.increment = increment;
		this.qualifier = qualifier;
		this.all = major + "." + minor + "." + increment + (qualifier == null ? "" : "-" + qualifier);
		this.ok = true;
	}

	public String toString() {
		return all;
	}

	public void toString(StringBuilder sb) {
		sb.append(all);
	}

}
