package aQute.maven;

import java.io.*;
import java.util.*;
import java.util.zip.*;

import aQute.lib.io.*;

class NexusIndexReader implements Iterator<IndexEntry>, Closeable {
	final DataInputStream	din;
	byte[]					buffer;
	boolean					hasnext;
	IndexEntry				index;
	boolean					debug	= false;

	public NexusIndexReader(InputStream in) throws IOException {
		GZIPInputStream zin = new GZIPInputStream(in);
		din = new DataInputStream(zin);
		din.skip(9);
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException("does not support remove");
	}

	@Override
	public IndexEntry next() {
		if (hasNext()) {
			hasnext = false;
			return index;
		} else
			throw new NoSuchElementException();
	}

	@Override
	public boolean hasNext() {
		if (hasnext)
			return true;

		index = new IndexEntry();
		while (true) {
			try {
				int columns = din.readInt();
				for (int i = 0; i < columns; i++) {
					/* int tag = */din.readByte();
					String key = readString(false);
					if ("1".equals(key)) {
						index.sha1 = readString(true);
					} else if ("u".equals(key)) {
						String s[] = readString(true).split("\\|");
						index.groupId = s[0];
						index.artifactId = s[1];
						index.version = s[2];
						if (!"NA".equals(s[3]))
							index.classifier = s[3];
						if (s.length > 4) {
							index.packaging = s[4];
						}
					} else if ("d".equals(key)) {
						index.description = readString(true);
						if (debug)
							System.out.println(key + "=" + index.description);
					} else if ("m".equals(key)) {
						String m = readString(true);
						index.modified = Long.parseLong(m);
					} else if ("n".equals(key)) {
						index.name = readString(true);
					} else if ("i".equals(key)) {
						String s = readString(true);
						String parts[] = s.split("\\|");
						if (parts.length >= 3) {
							index.packaging = parts[0];
							index.date = Long.parseLong(parts[1]);
							index.size = Integer.parseInt(parts[2]);
							// not sure what the rest is?
						} else
							System.err.println("Error in i " + s);
					} else if (key.contains("Group")) {
						skipString(true);
					} else {
						String s = readString(true);
						index.headers.put(key, s);
						if (debug)
							System.out.println(key + "=" + s);
					}
				}
				if (index.groupId != null)
					return hasnext = true;
			}
			catch (EOFException e) {
				IO.close(din);
				return false;
			}
			catch (IOException e) {
				IO.close(din);
				throw new RuntimeException(e);
			}
		}
	}

	private String readString(boolean big) throws IOException {
		int l = big ? din.readInt() : din.readUnsignedShort();
		if (buffer == null || buffer.length < l)
			buffer = new byte[l];
		din.readFully(buffer, 0, l);
		String s = new String(buffer, 0, l, "UTF-8");
		return s;
	}

	private void skipString(boolean big) throws IOException {
		int l = big ? din.readInt() : din.readUnsignedShort();
		din.skipBytes(l);
	}

	@Override
	public void close() throws IOException {
		din.close();
	}

	public void setDebug() {
		debug = true;
	}
}
